/*
 * A page mapper providing copied memory pages or deferring to another page
 * mapper to satisfy file accesses.
 *
 * Copyright (C) 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <list>

#include <fsserver/access_map.h>
#include <fsserver/generic_page_mapper.h>
#include <mem/memory_incremental.h>



/* A special flexpage class for copied flexpages. */

class CopiedFlexpage : public Flexpage
{
public:
    explicit CopiedFlexpage(Region *region = NULL);
};



/* A file mapper, associating flexpages with file regions. */

class CopiedPageMapper : public GenericPageMapper
{
protected:
    GenericPageMapper *_mapper;
    offset_t _size;

    /* Copied page support. */

    AccessMap _map;
    MemoryIncremental _memory;
    std::list<Flexpage *> _queue;

    /* Internal flexpage retrieval methods. */

    Flexpage *find(offset_t offset);

    Flexpage *replicate_flexpage(offset_t offset, Flexpage *flexpage);

public:
    explicit CopiedPageMapper(GenericPageMapper *mapper);

    virtual ~CopiedPageMapper();

    /* Interface for the pager, implementing GenericPageMapper. */

    Flexpage *get(offset_t offset, map_flags_t flags);

    void queue(Flexpage *flexpage);

    void flush_all(offset_t start, offset_t size);

    offset_t get_data_size();

    void set_data_size(offset_t size);
};

// vim: tabstop=4 expandtab shiftwidth=4
